#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2022 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Font Tools/Embossible v X.XX
#
#   V 0.00 Test versions
#   v 1.00 Options to create soft emboss appearance to characters to use as a base for other tools that add decorative faces
#   v 1.01 Removed leftover channel from saved selections
#   v 1.02 Initial inflate emboss testing
#   v 1.03 Inflate emboss added with options to apply gloss finish
#   v 1.04 Fixed small bug in Inflate option and added 'Preview character to new image' option 



from gimpfu import *
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkgrey = gimpcolor.RGB(50, 50, 50)
midgrey = gimpcolor.RGB(125, 125, 125)



def do_stuff(image, layer, emboss_style, steps, ridge_blur, ridge_grey, flattened_opacity, do_edges, depth, azimuth, elevation, inflate_style, inflate_strength, artifact_radius, artifact_delta, do_preview, preview_char):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
    blur_grey = ridge_grey
    emboss_grey = ridge_grey
    ridge_grey = float(ridge_grey/100)
    new_ridge_colour = (ridge_grey, ridge_grey, ridge_grey)
	
    layer_names_list = image.layers
	
    if preview_char == "": preview_char = "A"
    preview_char = preview_char + ".png"
	
    # check that the preview character is present or use another
    if preview_char in str(layer_names_list):
        prev_pos = str(layer_names_list).find(preview_char)
        prev_loop = str(layer_names_list).count(",",0,prev_pos)
    else:
        loop = 0
        preview_char = pdb.gimp_item_get_name(layer_names_list[loop])
        while preview_char=="space.png" or (preview_char not in str(layer_names_list[loop])):
            loop += 1
            preview_char = pdb.gimp_item_get_name(layer_names_list[loop])

    preview_char = preview_char.decode('utf-8')	######## Added to catch higher value unicodes
	
    dummy = 0
    active_layer_id = pdb.gimp_image_get_active_layer(image)
	
	
    loop_ptr = 1
    first_char_layer_name = "none"
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    original_active_layer = image.active_layer
	
		
    num_multi_layers,other_layers,num_char_layers = 0,0,0
    visi_loop, first_char_layer  = 0, -1
	
    if do_preview:
        # copy the preview character and paste it as a  new image
		
        preview_layer = pdb.gimp_image_get_layer_by_name(image, preview_char)
        success = pdb.gimp_edit_copy(preview_layer)
        preview_image = pdb.gimp_edit_paste_as_new_image(preview_layer)
            
        pdb.gimp_image_undo_group_start(preview_image) ##############
		
        preview_display = pdb.gimp_display_new(preview_image)

        num_preview_layers, preview_layer_ids = pdb.gimp_image_get_layers(preview_image)
        preview_layer_names_list = preview_image.layers
		
        loop = 0
        layer_id = preview_layer_ids[loop]
        layer_name = preview_layer_names_list[loop]
		
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(preview_layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')
        #pdb.gimp_message("display_layer_name = " + str(display_layer_name) )
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(preview_image, layer_name)
        active_layer_id = pdb.gimp_image_get_active_layer(preview_image)
		
	
        if emboss_style == 0:
            active_layer_id = proc_blurred(preview_image, active_layer_id, blur_grey, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, display_layer_name, flattened_opacity, do_edges, emboss_grey, emboss_style) # flat emboss
		
			
        elif emboss_style == 1:
            peaked_in = False
            active_layer_id = proc_ridged(preview_image, active_layer_id, new_ridge_colour, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, peaked_in, display_layer_name, do_edges, emboss_grey, emboss_style) # peak emboss
			

        elif emboss_style == 2:
            peaked_in = True
            active_layer_id = proc_ridged(preview_image, active_layer_id, new_ridge_colour, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, peaked_in, display_layer_name, do_edges, emboss_grey, emboss_style) # valley emboss
		
	
        else:
            peaked_in = False
            active_layer_id = proc_inflate(preview_image, active_layer_id, new_ridge_colour, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, peaked_in, display_layer_name, do_edges, emboss_grey, emboss_style, inflate_style, inflate_strength) # inflate emboss
			
			
        pdb.gimp_item_set_visible(active_layer_id, True)
		
        # Set end undo point for preview image
        pdb.gimp_image_undo_group_end(preview_image)

    else:
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            # check to see if the current layer is a custom font character layer
            test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
			
            if test_for_custom_layer:
                num_char_layers += 1
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
                    layer_names_list[visi_loop].set_offsets(0, 0)
				
            
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
            visi_loop += 1
		
		
        while loop >= 0:
	        # get info about layer we are working on for custom font mode
            layer_id = layer_ids[loop]
            layer_name = layer_names_list[loop]
	
            # cater for unicode characters above normal range
            display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
            display_layer_name = display_layer_name.decode('utf-8')	
	
            # check we have the layer id of the currently active layer
            pdb.gimp_image_set_active_layer(image, layer_name)
            layer_id = pdb.gimp_image_get_active_layer(image)

            # check to see if the current layer is a custom font character layer
            test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
	
	
            if test_for_custom_layer:
                # ensure we have the active layer and make it visible
                active_layer_id = pdb.gimp_image_get_active_layer(image)
                pdb.gimp_item_set_visible(active_layer_id, True)
	
	
	
	
                if emboss_style == 0:
                    active_layer_id = proc_blurred(image, active_layer_id, blur_grey, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, display_layer_name, flattened_opacity, do_edges, emboss_grey, emboss_style) # flat emboss
		
			
                elif emboss_style == 1:
                    peaked_in = False
                    active_layer_id = proc_ridged(image, active_layer_id, new_ridge_colour, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, peaked_in, display_layer_name, do_edges, emboss_grey, emboss_style) # peak emboss
			

                elif emboss_style == 2:
                    peaked_in = True
                    active_layer_id = proc_ridged(image, active_layer_id, new_ridge_colour, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, peaked_in, display_layer_name, do_edges, emboss_grey, emboss_style) # valley emboss
		
	
                else:
                    peaked_in = False
                    active_layer_id = proc_inflate(image, active_layer_id, new_ridge_colour, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, peaked_in, display_layer_name, do_edges, emboss_grey, emboss_style, inflate_style, inflate_strength) # inflate emboss
				
            loop_ptr += 1
            loop = num_layers - loop_ptr
		
		
        if first_char_layer_name <> "none": # the character layer to make visible on completion
            first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            pdb.gimp_item_set_visible(first_char_layer_id, True)
            pdb.gimp_image_set_active_layer(image,first_char_layer_id)

    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

def proc_inflate(image, layer, blur_grey, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, peaked_in, display_layer_name, do_edges, emboss_grey, emboss_style, inflate_style, inflate_strength):
	
    # Set start undo point
    #pdb.gimp_image_undo_group_start(image)	
    counter = 0
    pdb.gimp_image_select_item(image, 2, layer)
    while ( (pdb.gimp_selection_is_empty(image) == False)):
        pdb.gimp_selection_shrink(image, steps)
        counter += 1
	
    loop = 0
    fill_value = (255 / counter)
    factor = 5
    greycolour = int(255/factor) # set initial colour to new starting colour
    start_inc = int(255/factor)
    fill_value = int((255-(255/factor)) / counter) # increment for fill colour spread over smaller range
    fill_inc = fill_value
    pdb.gimp_image_select_item(image, 2, layer)
    pdb.gimp_selection_grow(image, 1)
    original_channel = pdb.gimp_selection_save(image)
    pdb.gimp_item_set_name(original_channel, "First saved selection")
    pdb.gimp_drawable_edit_clear(layer)
    if peaked_in:
        greycolour = 255
    else:
        greycolour = start_inc
    grey = gimpcolor.RGB(greycolour, greycolour, greycolour)
    pdb.gimp_context_set_foreground(grey)

    pdb.gimp_edit_fill(layer, 0)
    #pdb.gimp_image_freeze_layers(image)
    while loop<(counter-1):
	
        if peaked_in:
            greycolour = greycolour - (fill_inc)
        else:
            greycolour = greycolour + (fill_inc)
        pdb.gimp_selection_shrink(image, steps)
        # copy layer and add to image
        temp_copy = pdb.gimp_layer_new(image, layer.width, layer.height, 1, "temp copy", 100, 0, )
        image.add_layer(temp_copy, -1)
        grey = gimpcolor.RGB(greycolour, greycolour, greycolour)
        pdb.gimp_context_set_foreground(grey)

        pdb.gimp_edit_fill(temp_copy, 0)
        channel = pdb.gimp_selection_save(image)

        pdb.gimp_selection_none(image)
        pdb.gimp_image_select_item(image, 2, original_channel)
        pdb.plug_in_gauss_iir(image, temp_copy, ridge_blur, True, True) # blur the layer
        pdb.gimp_selection_none(image)
        pdb.gimp_image_select_item(image, 2, channel)
        pdb.gimp_image_remove_channel(image, channel)
        loop += 1
	
    pdb.gimp_selection_none(image)
	
    layer = pdb.gimp_image_merge_visible_layers(image, 0)
    # copy treated layer and add to image
    temp_copy = pdb.gimp_layer_copy(layer, False)
    image.add_layer(temp_copy, -1)
    pdb.gimp_layer_set_mode(temp_copy, 23) # Overlay mode to accentuate shades of grey
	
    if inflate_style == 0: #normal gloss with range of curves for strength option
        if inflate_strength>0:
            if inflate_strength == 4:
                curves_list = [0.0,0.0,(1.0/255)*92,(1.0/255)*98,(1.0/255)*93,1.0,1.0,1.0]
            if inflate_strength == 3:
                curves_list = [0.0,0.0,(1.0/255)*92,(1.0/255)*98,(1.0/255)*128,1.0,1.0,1.0]
            if inflate_strength == 2:
                curves_list = [0.0,0.0,(1.0/255)*92,(1.0/255)*98,(1.0/255)*160,1.0,1.0,1.0 ]
            if inflate_strength == 1:
                curves_list = [0.0,0.0,(1.0/255)*92,(1.0/255)*98,(1.0/255)*192,(1.0/255)*225,1.0,1.0]
	
            pdb.gimp_drawable_curves_spline(temp_copy, 0, len(curves_list), curves_list)
            pdb.gimp_layer_set_mode(temp_copy, 28)
			
			
    if inflate_style == 1: #halo gloss with range of curves for strength option
        if inflate_strength>0:
            if inflate_strength == 4:
                curves_list = [0.0,0.0,(1.0/255)*65,(1.0/255)*34,(1.0/255)*95,(1.0/255)*94,(1.0/255)*128,1.0,1.0,1.0]
            if inflate_strength == 3:
                curves_list = [0.0,0.0,(1.0/255)*65,(1.0/255)*34,(1.0/255)*95,(1.0/255)*94,(1.0/255)*160,1.0,1.0,1.0]
            if inflate_strength == 2:
                curves_list = [0.0,0.0,(1.0/255)*65,(1.0/255)*34,(1.0/255)*95,(1.0/255)*94,(1.0/255)*190,1.0,1.0,1.0]
            if inflate_strength == 1:
                curves_list = [0.0,0.0,(1.0/255)*65,(1.0/255)*34,(1.0/255)*95,(1.0/255)*94,(1.0/255)*223,1.0,1.0,1.0]
	
            pdb.gimp_drawable_curves_spline(temp_copy, 0, len(curves_list), curves_list)
            pdb.gimp_layer_set_mode(temp_copy, 28)

    if inflate_strength>0: pdb.gimp_drawable_edit_clear(layer) # clear the original layer contents if using a gloss effect

    layer = pdb.gimp_image_merge_down(image, temp_copy, 0)
    pdb.gimp_item_set_visible(layer, False)
	
    #return temp_copy
    return layer
	
    # Set end undo point
    #pdb.gimp_image_undo_group_end(image)
	
	
######################################################################################################################################

def proc_blurred(image, layer, blur_grey, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, display_layer_name, opacity, do_edges, emboss_grey, emboss_style): # flattened emboss
	
    # Set start undo point
    #pdb.gimp_image_undo_group_start(image)	
    counter = 0
    pdb.gimp_image_select_item(image, 2, layer)
    while ( (pdb.gimp_selection_is_empty(image) == False)):
        pdb.gimp_selection_shrink(image, steps)
        counter += 1
	
    loop = 0
    pdb.gimp_image_select_item(image, 2, layer)
    pdb.gimp_selection_grow(image, 1)
    original_channel = pdb.gimp_selection_save(image)
    pdb.gimp_drawable_edit_clear(layer)
	
    fill_value = (255 / counter) # increment for fill colour
    fill_inc = fill_value
	#### initial fill colour
    greycolour = 0 # black
    greycolour = int(blur_grey*2.5) # take colour from user choice
    grey = gimpcolor.RGB(greycolour, greycolour, greycolour)
    pdb.gimp_context_set_foreground(grey)

    pdb.gimp_selection_grow(image, (steps/2))
    pdb.gimp_edit_fill(layer, 0)
    pdb.gimp_layer_set_opacity(layer,opacity)
    #pdb.plug_in_gauss_iir(image, layer, ridge_blur, True, True) # blur the layer
    pdb.gimp_image_freeze_layers(image)
    while loop<(counter-1):
	
        pdb.gimp_selection_shrink(image, steps)
        # copy layer and add to image
        temp_copy = pdb.gimp_layer_new(image, layer.width, layer.height, 1, "temp copy", 100, 0, )
        image.add_layer(temp_copy, -1)
        #greycolour = int(fill_value)
        #grey = gimpcolor.RGB(greycolour, greycolour, greycolour)
        #pdb.gimp_context_set_foreground(grey)

        pdb.gimp_edit_fill(temp_copy, 0)
        channel = pdb.gimp_selection_save(image)

        pdb.gimp_selection_none(image)
        pdb.gimp_image_select_item(image, 2, original_channel)
        pdb.plug_in_gauss_iir(image, temp_copy, ridge_blur, True, True) # blur the layer
        pdb.gimp_layer_set_opacity(temp_copy,opacity)
        pdb.gimp_selection_none(image)
        pdb.gimp_image_select_item(image, 2, channel)
        pdb.gimp_image_remove_channel(image, channel)

        loop += 1
	
    pdb.gimp_selection_none(image)
	
    layer = pdb.gimp_image_merge_visible_layers(image, 0)

    temp_copy = pdb.gimp_layer_new(image, layer.width, layer.height, 1, "temp copy", 100, 0, )
    image.add_layer(temp_copy, -1)
    pdb.gimp_context_set_foreground(grey)

    pdb.gimp_edit_fill(temp_copy, 0)
	
    pdb.gimp_image_thaw_layers(image)
	
    pdb.gimp_image_select_item(image, 2, original_channel)
    pdb.gimp_selection_shrink(image, 1)
    # invert selection
    pdb.gimp_selection_invert(image)
    # trim away pixels
    successful = pdb.gimp_edit_cut(temp_copy)
    # deselect
    pdb.gimp_selection_none(image)

    pdb.plug_in_bump_map(image, temp_copy, layer, azimuth, elevation, depth, 0, 0, 0, 0, True, False, 0)
    pdb.plug_in_sel_gauss(image, temp_copy, artifact_radius, artifact_delta)
	
    pdb.gimp_image_remove_layer(image, layer)
    pdb.gimp_image_remove_channel(image, original_channel)
	
    
    pdb.gimp_item_set_name(temp_copy, display_layer_name)
	
	
    if do_edges: 
        tempcopy = proc_emboss_edges(image, temp_copy, white, emboss_grey, depth, elevation, azimuth, 10, 47, artifact_radius, artifact_delta, emboss_style)
	
	
    pdb.gimp_item_set_visible(temp_copy, False)
	
    return temp_copy
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)
	
######################################################################################################################################

def proc_ridged(image, layer, blur_grey, steps, ridge_blur, azimuth, elevation, depth, artifact_radius, artifact_delta, peaked_in, display_layer_name, do_edges, emboss_grey, emboss_style):
	
    # Set start undo point
    #pdb.gimp_image_undo_group_start(image)	
    counter = 0
    pdb.gimp_image_select_item(image, 2, layer)
    while ( (pdb.gimp_selection_is_empty(image) == False)):
        pdb.gimp_selection_shrink(image, steps)
        counter += 1
	
    loop = 0
    fill_value = (255 / counter)
    factor = 5
    greycolour = int(255/factor) # set initial colour to new starting colour
    start_inc = int(255/factor)
    fill_value = int((255-(255/factor)) / counter) # increment for fill colour spread over smaller range
    fill_inc = fill_value
    pdb.gimp_image_select_item(image, 2, layer)
    pdb.gimp_selection_grow(image, 1)
    original_channel = pdb.gimp_selection_save(image)
    pdb.gimp_item_set_name(original_channel, "First saved selection")
    pdb.gimp_drawable_edit_clear(layer)
    if peaked_in:
        greycolour = 255
    else:
        greycolour = start_inc
    grey = gimpcolor.RGB(greycolour, greycolour, greycolour)
    pdb.gimp_context_set_foreground(grey)

    pdb.gimp_edit_fill(layer, 0)
    #pdb.gimp_image_freeze_layers(image)
    while loop<(counter-1):
	
        if peaked_in:
            greycolour = greycolour - (fill_inc)
        else:
            greycolour = greycolour + (fill_inc)
        pdb.gimp_selection_shrink(image, steps)
        # copy layer and add to image
        temp_copy = pdb.gimp_layer_new(image, layer.width, layer.height, 1, "temp copy", 100, 0, )
        image.add_layer(temp_copy, -1)
        grey = gimpcolor.RGB(greycolour, greycolour, greycolour)
        pdb.gimp_context_set_foreground(grey)

        pdb.gimp_edit_fill(temp_copy, 0)
        channel = pdb.gimp_selection_save(image)

        pdb.gimp_selection_none(image)
        pdb.gimp_image_select_item(image, 2, original_channel)
        pdb.plug_in_gauss_iir(image, temp_copy, ridge_blur, True, True) # blur the layer
        pdb.gimp_selection_none(image)
        pdb.gimp_image_select_item(image, 2, channel)
        pdb.gimp_image_remove_channel(image, channel)
        loop += 1
	
    pdb.gimp_selection_none(image)
	
    layer = pdb.gimp_image_merge_visible_layers(image, 0)

    temp_copy = pdb.gimp_layer_new(image, layer.width, layer.height, 1, "temp copy", 100, 0, )
    image.add_layer(temp_copy, -1)
    pdb.gimp_context_set_foreground(blur_grey)

    pdb.gimp_edit_fill(temp_copy, 0)
	
    pdb.gimp_layer_set_mode(temp_copy, 47)
    layer = pdb.gimp_image_merge_down(image, temp_copy, 0)
	
	
    temp_copy = pdb.gimp_layer_new(image, layer.width, layer.height, 1, "temp copy", 100, 0, )
    image.add_layer(temp_copy, -1)
    pdb.gimp_context_set_foreground(blur_grey)

    pdb.gimp_edit_fill(temp_copy, 0)
	
    #pdb.gimp_image_thaw_layers(image)
	
    
    pdb.plug_in_bump_map(image, temp_copy, layer, azimuth, elevation, depth, 0, 0, 0, 0, True, False, 0)
    pdb.gimp_image_select_item(image, 2, original_channel)
    pdb.gimp_selection_shrink(image, 1)
    # invert selection
    pdb.gimp_selection_invert(image)
    # trim away pixels
    successful = pdb.gimp_edit_cut(temp_copy)
    # deselect
    pdb.gimp_selection_none(image)
    pdb.gimp_image_remove_channel(image, original_channel)
   
    pdb.plug_in_sel_gauss(image, temp_copy, artifact_radius, artifact_delta)	
	
	
    pdb.gimp_image_remove_layer(image, layer)
    pdb.gimp_item_set_name(temp_copy, display_layer_name)
	
    if do_edges: 
        tempcopy = proc_emboss_edges(image, temp_copy, white, emboss_grey, depth, elevation, azimuth, 10, 47, artifact_radius, artifact_delta, emboss_style)
		
		
	
    pdb.gimp_item_set_visible(temp_copy, False)
	
    return temp_copy
	
	
    # Set end undo point
    #pdb.gimp_image_undo_group_end(image)
	
######################################################################################################################################

	
#def proc_emboss_edges(image, active_layer, grey, white, effect_width, effect_elevation, effect_azimuth, emboss_blur, effect_mode_ptr, blur_radius, blur_delta):
def proc_emboss_edges(image, active_layer, white, emboss_grey, effect_width, effect_elevation, effect_azimuth, emboss_blur, effect_mode_ptr, blur_radius, blur_delta, emboss_style):

    if emboss_style <> 2:
        effect_azimuth = (effect_azimuth + 180) % 360
	
    effect_elevation = 45
    emboss_amount = emboss_grey
    effect_width = 10
    emboss_amount = 100
    grey_value = int(emboss_amount)
    grey = gimpcolor.RGB(grey_value, grey_value, grey_value)
    if effect_width < 1: effect_width = 1
    work_layer = active_layer # ie character layer
    work_layer_name = work_layer.name # ie character layer id
    width = work_layer.width
    height = work_layer.height
    colour = gimp.get_background()
    emboss_mode = effect_mode_ptr
	
    layer_blur = image.height * 0.015
    if layer_blur < 1: layer_blur = 1	
	
    pdb.gimp_context_set_foreground(grey)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_image_select_item(image, 2, work_layer)
	
    emboss_layer = gimp.Layer(image, "Emboss Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the bump map
    image.add_layer(emboss_layer, -1)
    emboss_layer.fill(1)
    pdb.gimp_drawable_edit_fill(emboss_layer, 0)
	
	
    pdb.gimp_context_set_foreground(grey)
    pdb.gimp_context_set_background(white)
	
	
	
    pdb.gimp_selection_none(image) ####
    pdb.plug_in_gauss_iir(image, emboss_layer, emboss_blur, True, True) # blur the bump map
	
    other_layers = 1
    bumpmap_layer_name = "Emboss Layer"
    bumpmap_layer = emboss_layer
	
    #temp values
    bump_waterlevel = 0
    bump_ambient = 0
    waterlevel = 0
    ambient = 0
    #end temp values
	
    bump_depth = effect_width
	
    work_layer = active_layer
    work_layer_name = work_layer.name
    width = work_layer.width
    height = work_layer.height
		
    if waterlevel > 0:
        waterlevel = waterlevel/100
        waterlevel = 1.0 - waterlevel
    else:
        waterlevel = 0.0
			
    if ambient > 0:
        ambient = ambient/100
        ambient = 1.0 - ambient
    else:
        ambient = 0.0        

    pdb.plug_in_bump_map(image, work_layer, emboss_layer, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
	
    image.remove_layer(emboss_layer)
    pdb.gimp_image_select_item(image, 2, work_layer)
    work_layer = image.active_layer
		
    pdb.gimp_selection_none(image)
    active_layer = image.active_layer 
    return active_layer	
	
######################################################################################################################################

def proc_pillow(image, layer, new_f_colour, blur, azimuth, elevation, depth, artifact_radius, artifact_delta):

  
    # Set start undo point
    #pdb.gimp_image_undo_group_start(image)	
	
    width = layer.width
    height = layer.height  


    emboss_layer = gimp.Layer(image, "Emboss Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the bump map
    image.add_layer(emboss_layer, -1)
	

    # alpha select
    pdb.gimp_image_select_item(image, 2, layer)
    pdb.gimp_context_set_foreground(new_f_colour)
    pdb.gimp_drawable_edit_fill(emboss_layer, 0)


	
    pdb.gimp_selection_none(image)
	
	
    pdb.plug_in_gauss_iir(image, emboss_layer, blur, True, True) # blur the layer
	
	
    pdb.gimp_image_select_item(image, 2, layer)
    pdb.gimp_selection_invert(image)
	
    # trim away pixels
    successful = pdb.gimp_edit_cut(emboss_layer)
    pdb.gimp_selection_none(image)
	
	
    # copy layer and add to image
    first_copy = pdb.gimp_layer_new_from_drawable(emboss_layer, image)
    image.add_layer(first_copy, -1)
	
    layer = pdb.gimp_image_merge_down(image, emboss_layer, 0)
    layer = pdb.gimp_image_merge_down(image, first_copy, 0)
	
	####
	
	
    temp_copy = pdb.gimp_layer_new(image, layer.width, layer.height, 1, "temp copy", 100, 0, )
    image.add_layer(temp_copy, -1)
    pdb.gimp_context_set_foreground(new_f_colour)

    pdb.gimp_edit_fill(temp_copy, 0)
	
	
    pdb.plug_in_bump_map(image, temp_copy, layer, azimuth, elevation, depth, 0, 0, 0, 0, True, False, 0)
    pdb.gimp_image_select_item(image, 2, layer)
    pdb.gimp_selection_shrink(image, 1)
    # invert selection
    pdb.gimp_selection_invert(image)
    # trim away pixels
    successful = pdb.gimp_edit_cut(temp_copy)
    # deselect
    pdb.gimp_selection_none(image)
    pdb.plug_in_sel_gauss(image, temp_copy, artifact_radius, artifact_delta)
	
    pdb.gimp_image_remove_layer(image, layer)
	
	
	
	#####
	
	
    # Set end undo point
    #pdb.gimp_image_undo_group_end(image)
	
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Embossible_v1_04",
    "Provides options for embossing \nCustom Font characters",
    "Provides options for embossing \nCustom Font characters",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2022",
    "<Image>/Filters/Custom Font Tools/Custom Font Embossible v 1.04...",
    "RGB*",
    [
    (PF_OPTION, "type_option", "Emboss style: ", 0, ("Flat", "Peak","Valley", "Inflated")),
    (PF_SPINNER, "pf_ridge_int", "Section width: ", 10,(1,100,1)),
    (PF_SPINNER, "pf_r_blur", "Section blur %: ", 10,(0,90,1)),
    (PF_SPINNER, "pf_rlightness", "Final shading (Dark to Light): ", 50,(0,90,1)),
    (PF_SPINNER, "pf_opacity", "Flattened section opacity: ", 75,(10,90,5)),
    (PF_TOGGLE, "pf_finalemboss", "Emboss edges: ", 1),
    (PF_SPINNER, "pf_emboss_depth", "Emboss Depth: ", 40,(0,65,1)),
    (PF_SLIDER, "pf_azimuth", "Emboss Azimuth:", 135, (0, 359, 1)),
    (PF_SLIDER, "pf_elevation", "Emboss Elevation: ", 20, (0, 179, 1)),
    (PF_OPTION, "inflate_option", "Inflated gloss style: ", 0, ("Normal gloss", "Halo gloss")),
    (PF_SPINNER, "inflate_strength", "Inflated gloss strength: ", 0,(0,4,1)),
    (PF_SPINNER, "pf_blur_radius", "Artifact blur radius: ", 20.00,(0.00,100.00,0.1)),
    (PF_SPINNER, "pf_blur_delta", "Artifact blur delta amount: ", 100,(0,255,1)),
    (PF_TOGGLE, "Preview_option", "Preview single character\n(to a new image): ", 0),
    (PF_STRING, "preview_string", "Preview character: ", "A"),
    ],
    [],
    do_stuff)

main()
